/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2005-2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: appletswinclass.c,v 1.3 2006/12/20 13:34:13 fab Exp $
 */

#include "ambient.h"

#if USE_TRAY_LIB
/* public */

/* private */
#include "ambient_cat.h"
#include "mui_func.h"
#include "screen.h"
#include "threads.h"
#include "methodstack.h"

struct Data {
	ULONG viewid;
	APTR wlist;
	APTR blist;
};


DEFNEW
{
	struct Data *data;
	APTR bt_ok, bt_cancel, bt_right, bt_left, lv_white, lv_black, lst_white, lst_black;

	obj = DoSuperNew(cl, obj,
		MUIA_Window_Screen, get_screen(),
		MUIA_Window_ScreenTitle, screentitle,
		MUIA_Window_Title, GSI(MSG_APPLETSWINCLASS_TITLE),
		MUIA_Window_LeftEdge, MUIV_Window_LeftEdge_Moused,
		MUIA_Window_TopEdge, MUIV_Window_TopEdge_Moused,
		MUIA_Window_ShowIconify, FALSE,
		MUIA_Window_ShowPrefs, FALSE,
		MUIA_Window_ShowJump, FALSE,
		MUIA_Window_ShowAbout, FALSE,
		MUIA_Window_NoMenus, TRUE,
		MUIA_Window_CloseGadget, FALSE,
		WindowContents, VGroup,
			Child, HGroup,

				Child, HGroup, MUIA_Weight, 2, GroupFrameT(GSI(MSG_APPLETSWINCLASS_GROUP_WHITELIST)),
					Child, lv_white = ListviewObject,
						MUIA_Listview_List, lst_white = ListObject,
							MUIA_List_ConstructHook, MUIV_List_ConstructHook_String,
							MUIA_List_DestructHook, MUIV_List_DestructHook_String,
							InputListFrame,
							End,
					End,
				End,

				Child, VGroup, MUIA_Weight, 1,
					Child, VSpace(0),
					Child, bt_right = SimpleButton("\033I[6:30]"),
					Child, bt_left = SimpleButton("\033I[6:31]"),
					Child, SimpleButton("\033I[6:30]\033I[6:30]\033I[6:30]"),
					Child, SimpleButton("\033I[6:31]\033I[6:31]\033I[6:31]"),
					Child, VSpace(0),
				End,

				Child, HGroup, MUIA_Weight, 2, GroupFrameT(GSI(MSG_APPLETSWINCLASS_GROUP_BLACKLIST)),
					Child, lv_black = ListviewObject,
							MUIA_Listview_List, lst_black = ListObject,
							MUIA_List_ConstructHook, MUIV_List_ConstructHook_String,
							MUIA_List_DestructHook, MUIV_List_DestructHook_String,
							InputListFrame,
							End,
					End,
				End,
			End,

			Child, HGroup,
				Child, bt_ok     = MUICreateButton( MSG_APPLETSWINCLASS_USE   , NULL),
/* removed button, because didn't work anyway (geit)
			  Child, bt_cancel = MUICreateButton( MSG_APPLETSWINCLASS_CANCEL, NULL),  */
			End,
		End,
	End;

	if (!obj)
	{
		return (NULL);
	}

	data = INST_DATA(cl, obj);
	data->wlist = lst_white;
	data->blist = lst_black;

	DoMethod(bt_ok    , MUIM_Notify, MUIA_Pressed, FALSE, obj, 3, MUIM_Set, MUIA_Window_Open, FALSE);
/* removed button, because pushing had no other effect then closing the window (geit)
	DoMethod(bt_cancel, MUIM_Notify, MUIA_Pressed, FALSE, obj, 3, MUIM_Set, MUIA_Window_Open, FALSE); */
	DoMethod(bt_right, MUIM_Notify, MUIA_Pressed, FALSE, obj, 3, MM_Appletswin_MoveApplet, MV_Appletswin_WhiteList, MV_Appletswin_ActiveEntry);
	DoMethod(bt_left, MUIM_Notify, MUIA_Pressed, FALSE, obj, 3, MM_Appletswin_MoveApplet, MV_Appletswin_BlackList, MV_Appletswin_ActiveEntry);
	
    return ((ULONG)obj);
}


DEFGET
{
	switch (msg->opg_AttrID)
	{
		case MA_Window_ID:
			*msg->opg_Storage = 0;
			return (TRUE);

		case MA_Window_Type:
			*msg->opg_Storage = MV_Window_Type_Select;
			return (TRUE);
	}
	return (DOSUPER);
}

static int check_dupe(APTR list, STRPTR entry);

static int check_dupe(APTR list, STRPTR entry)
{
	int i;

	for (i = 0;;i++)
	{
		STRPTR e;

		DoMethod(list, MUIM_List_GetEntry, i, &e);
		
        if (!e)
			break;

		if (strcmp(e, entry) == 0)
			return (TRUE);
	}

	return (FALSE);
}


DEFSMETHOD(Appletswin_AddApplet)
{
	GETDATA;
	APTR list = NULL;

    if (msg->list == MV_Appletswin_WhiteList)
	{
		list = data->wlist;
		if (check_dupe(data->blist, (STRPTR)msg->applet))
			return (FALSE);
    }
	else if (msg->list == MV_Appletswin_BlackList)
	{
		list = data->blist;
	}

	if (list && !check_dupe(data->wlist, (STRPTR)msg->applet) && !check_dupe(data->blist, (STRPTR)msg->applet))
	{
		DoMethod(list, MUIM_List_InsertSingle, msg->applet, MUIV_List_Insert_Sorted);
	}

	return (TRUE);
}

DEFSMETHOD(Appletswin_MoveApplet)
{
	APTR src = NULL, dest = NULL;
	STRPTR applet;
	int i;
	int entry = 0;
	GETDATA;

	if (msg->srclist == MV_Appletswin_WhiteList)
	{
		src = data->wlist;
		dest = data->blist;
	}
	else if (msg->srclist == MV_Appletswin_BlackList)
	{
		src = data->blist;
		dest = data->wlist;
	}

	if (src)
	{
		if (msg->applet == MV_Appletswin_ActiveEntry)
		{
			get(src, MUIA_List_Active, &entry);
		}

		if (entry >= 0)
		{
			DoMethod(src, MUIM_List_GetEntry, entry, &applet);
			DoMethod(dest, MUIM_List_InsertSingle, applet, MUIV_List_Insert_Sorted);
			DoMethod(src, MUIM_List_Remove, entry);
		}
	}

	return (0);
}

DEFSMETHOD(Appletswin_GetList)
{
	GETDATA;
	ULONG len = 0;
	STRPTR list = NULL, dest;
	APTR src = NULL;
    int i;

	if (msg->list == MV_Appletswin_WhiteList)
	{
		DB(("GetList:White\n"));
		src = data->wlist;
	}
	else if (msg->list == MV_Appletswin_BlackList)
	{
		DB(("GetList:Black\n"));
		src = data->blist;
	}

	if (src)
	{
		for (i = 0;;i++)
		{
			STRPTR applet;

			DoMethod(src, MUIM_List_GetEntry, i, &applet);

			if (!applet)
				break;

			len += strlen(applet) + 1;
		}

		len += 1;

		list = malloc(len);
		dest = list;

		for (i = 0;;i++)
		{
			int l;
			STRPTR applet;

			DoMethod(src, MUIM_List_GetEntry, i, &applet);
		
			if (!applet)
				break;

			l = strlen(applet) + 1;

			memcpy(dest, applet, l);
			DB(("applet %s\n", applet));
            dest += l;
		}
	
		*dest = 0;
	}

	*((STRPTR *)msg->outlist) = list;
	*((ULONG *)msg->outlen)	= len;

	return (0);
}


BEGINMTABLE
DECNEW
DECGET
DECSMETHOD(Appletswin_AddApplet)
DECSMETHOD(Appletswin_MoveApplet)
DECSMETHOD(Appletswin_GetList)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Window, appletswinclass)

#endif
