/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: panelwinclass.c,v 1.9 2007/02/11 22:35:30 fab Exp $
 */

#include "ambient.h"

/* public */

/* private */
#include "mui_func.h"
#include "screen.h"
#include "prefs.h"
#include "threads.h"
#include "panelprefs.h"
#include "legacy.h"

#if USE_TRAY_LIB
#include <proto/tray.h>
#endif

struct Data {
	APTR grp;
	APTR pctx;
	ULONG panelwin_delete;
	ULONG attaching;
	ULONG attached;
#if USE_TRAY_LIB
    tray_server *ts;
#endif
};


DEFNEW
{
	struct Data *data;
	APTR grp;
	APTR pctx;
	struct TagItem *ti;
	UBYTE spec[128];

	if ( (ti = FindTagItem(MA_Panelwin_Prefspool, INITTAGS)) )
	{
		pctx = (APTR)ti->ti_Data;
	}
	else
	{
		pctx = prefspool_create(0);
	}
	
	if (!pctx)
	{
		PDB(("sigh, no prefspool\n"));
		return (NULL);
	}

	memset(spec, 0, sizeof(spec));

	panelprefs_fix(pctx);

	/*
	 * Set the defaults.
	 */
	setprefslong_default_ctx(pctx, DSI_PANELGROUP_SIZE, MV_Panelgroup_Size_Large);
	setprefslong_default_ctx(pctx, DSI_PANELGROUP_DEPTH, MV_Panelgroup_Depth_Normal);
	setprefslong_default_ctx(pctx, DSI_PANELGROUP_ISHORIZ, TRUE);
	setprefslong_default_ctx(pctx, DSI_PANELGROUP_DRAGGADGET_PLACEMENT, MV_Panelgroup_DragMode_LeftUp);
	setprefslong_default_ctx(pctx, DSI_PANELGROUP_ZIPPING_ENABLED, TRUE);
	setprefsstr_default_ctx(pctx, DSI_PANELGROUP_FRAMESPEC, "000000");

	setprefslong_default_ctx(pctx, DSI_PANELGROUP_POSMODE, 0);
	setprefslong_default_ctx(pctx, DSI_PANELGROUP_X, 400);
	setprefslong_default_ctx(pctx, DSI_PANELGROUP_Y, 300);
	setprefslong_default_ctx(pctx, DSI_PANELGROUP_LOCKED, FALSE);
	setprefsstr_default_ctx(pctx, DSI_PANELGROUP_IMAGESPEC, "2:m2");

	obj = DoSuperNew(cl, obj,
		MUIA_Window_Screen, get_screen(),
		MUIA_Window_ScreenTitle, screentitle,
		MUIA_Window_CloseGadget, FALSE,
		MUIA_Window_Borderless, TRUE,
		EnableExtensions ? MUIA_Window_PanelWindow : TAG_IGNORE, TRUE,
		MUIA_Window_CloseGadget, FALSE,
		MUIA_Window_DepthGadget, FALSE,
		MUIA_Window_SizeGadget, FALSE,
		MUIA_Window_DragBar, FALSE,
		MUIA_Window_BackfillHook, LAYERS_NOBACKFILL,
		MUIA_Window_LeftEdge, getprefslong_ctx(pctx, DSI_PANELGROUP_X),
		MUIA_Window_TopEdge, getprefslong_ctx(pctx, DSI_PANELGROUP_Y),
		WindowContents, VGroup, /* needed so we can have settable frames */
			InnerSpacing(0, 0),
			Child, grp = NewObject(getpanelgroupclass(), NULL,
				MA_Panelgroup_Locked, getprefslong_ctx(pctx, DSI_PANELGROUP_LOCKED),
				MA_Panelgroup_Size, getprefslong_ctx(pctx, DSI_PANELGROUP_SIZE),
				MA_Panelgroup_Horiz, getprefslong_ctx(pctx, DSI_PANELGROUP_ISHORIZ),
				MA_Panelgroup_DragMode, getprefslong_ctx(pctx, DSI_PANELGROUP_DRAGGADGET_PLACEMENT),
				MA_Panelgroup_Zipping, getprefslong_ctx(pctx, DSI_PANELGROUP_ZIPPING_ENABLED),
				MA_Panelgroup_FrameSpec, getprefsstr_ctx(pctx, DSI_PANELGROUP_FRAMESPEC),
				MA_Panelgroup_ImageSpec, getprefsstr_ctx(pctx, DSI_PANELGROUP_IMAGESPEC),
				MA_Panelgroup_AutoZip, getprefslong_ctx(pctx, DSI_PANELGROUP_AUTOZIP),
			End,
		End,
	End;

	if (!obj)
	{
		prefspool_delete(pctx);
		return (NULL);
	}

	data = INST_DATA(cl, obj);
	data->grp = grp;
	data->pctx = pctx;
	data->attached = getprefslong_ctx(pctx, DSI_PANELGROUP_POSMODE);

	set(data->grp, MA_Panelgroup_Depth, getprefslong_ctx(pctx, DSI_PANELGROUP_DEPTH));

	/*
	 * Prefs saving.
	 */
	DoMethod(data->grp, MUIM_Notify, MA_Panelgroup_HasChanged, TRUE,
		obj, 1, MM_Panelwin_SaveConfig
	);

	DoMethod(obj, MUIM_Notify, MA_Panelwin_Attached, MUIV_EveryTime,
		obj, 1, MM_Panelwin_SaveConfig
	);

	/*
	 * Close the panelgroup config win when switching to
	 * locked mode.
	 */
	DoMethod(data->grp, MUIM_Notify, MA_Panelgroup_Locked, TRUE,
		obj, 1, MM_Panelwin_CloseConfigwin
	);

	return ((ULONG)obj);
}


DEFGET
{
	GETDATA;

	switch (msg->opg_AttrID)
	{
		case MA_Window_ID:
			*msg->opg_Storage = prefspool_uid(data->pctx) + 0x10000;
			return (TRUE);

		case MA_Window_Path:
			*msg->opg_Storage = NULL; /* XXX: fix fix ! */
			return (TRUE);

		case MA_Window_Type:
			*msg->opg_Storage = MV_Window_Type_Panel;
			return (TRUE);
	
		case MA_Panelwin_Group:
			{
				GETDATA;
				*msg->opg_Storage = (ULONG)data->grp;
			}
			return (TRUE);

		case MA_Panelwin_Attached:
			{
				GETDATA;
				*msg->opg_Storage = data->attached;
			}
			return (TRUE);
	}
	return (DOSUPER);
}


DEFSET
{
	struct TagItem *ti;

	if ( (ti = FindTagItem(MA_Panelwin_Attached, INITTAGS)) )
	{
		GETDATA;

		data->attached = ti->ti_Data;

		if (data->attached)
		{
			DoMethod(obj, MM_Panelwin_AttachToBorder, FALSE);
		}
	}
	return (DOSUPER);
}


DEFTMETHOD(Panelwin_CloseConfigwin)
{
	APTR configwin;
	GETDATA;

	if ( (configwin = (APTR)getv(data->grp, MA_Panelgroup_Configwin)) )
	{
		DoMethod(configwin, MM_Panelconfigwin_Close);
	}
	return (0);
}


DEFTMETHOD(Panelwin_Close)
{
	GETDATA;
	ULONG deleting = FALSE;

	set(obj, MUIA_Window_Open, FALSE);

	DoMethod(obj, MM_Panelwin_CloseConfigwin);

	if (data->pctx)
	{
		if (data->panelwin_delete)
		{
			/* XXX: there could be some other pending jobs.. how to abort them properly ? */
			deleting = do_action(obj, TA_Panels_Delete,
				TT_Panels_Delete_Ctx, data->pctx,
			TAG_DONE);
		}
	}

	if (!deleting)
	{
		/* we close the window in MM_Thread_Finished */
		threads_abort(obj, NULL);
	}
	return (0);
}


DEFSMETHOD(Panelwin_Delete_Ok)
{
	switch (msg->butnum)
	{
		case 0: /* cancel */
			break;

		case 1: /* delete */
			{
				GETDATA;
				data->panelwin_delete = TRUE;
				DoMethod(app, MUIM_Application_PushMethod, obj, 1, MM_Panelwin_Close);
			}
			break;

		#ifdef DEBUG
		default:
			PDB(("no action\n"));
			break;
		#endif
	}
	return (0);
}


DEFTMETHOD(Panelwin_SaveConfig)
{
	GETDATA;
	APTR bctxt;

	setprefslong_ctx(data->pctx, DSI_PANELGROUP_SIZE, getv(data->grp, MA_Panelgroup_Size));
	setprefslong_ctx(data->pctx, DSI_PANELGROUP_DEPTH, getv(data->grp, MA_Panelgroup_Depth)),
	setprefslong_ctx(data->pctx, DSI_PANELGROUP_ISHORIZ, getv(data->grp, MA_Panelgroup_Horiz));
	setprefslong_ctx(data->pctx, DSI_PANELGROUP_POSMODE, data->attached);
	setprefslong_ctx(data->pctx, DSI_PANELGROUP_DRAGGADGET_PLACEMENT, getv(data->grp, MA_Panelgroup_DragMode));
	setprefslong_ctx(data->pctx, DSI_PANELGROUP_ZIPPING_ENABLED, getv(data->grp, MA_Panelgroup_Zipping));
	setprefslong_ctx(data->pctx, DSI_PANELGROUP_LOCKED, getv(data->grp, MA_Panelgroup_Locked));
	setprefslong_ctx(data->pctx, DSI_PANELGROUP_AUTOZIP, getv(data->grp, MA_Panelgroup_AutoZip));

	/*
	 * During window depth change, the window is closed
	 * during saving and would get 0/0 coordinates without
	 * the following check.
	 */

	if (getv(obj, MUIA_Window_Open))
	{
		setprefslong_ctx(data->pctx, DSI_PANELGROUP_X, getv(obj, MUIA_Window_LeftEdge));
		setprefslong_ctx(data->pctx, DSI_PANELGROUP_Y, getv(obj, MUIA_Window_TopEdge));
	}

	//dprintf("saving panel %p, lockmode %ld\n", data->grp, getv(data->grp, MA_Panelgroup_Locked));
	
	setprefsstr_ctx(data->pctx, DSI_PANELGROUP_FRAMESPEC, (STRPTR)getv(data->grp, MA_Panelgroup_FrameSpec));
	setprefsstr_ctx(data->pctx, DSI_PANELGROUP_IMAGESPEC, (STRPTR)getv(data->grp, MA_Panelgroup_ImageSpec));

	/*
	 * Now save the panel content.
	 */
	DoMethod(data->grp, MM_Panelgroup_SaveConfig, data->pctx);

	if ( (bctxt = prefspool_duplicate(data->pctx)) )
	{
		panelprefs_save(bctxt);
	}

	/* XXX */
	return (0);
}


#define CORNER_ATTRACTION 6 /* percent of the screen size */

DEFSMETHOD(Panelwin_AttachToBorder)
{
	GETDATA;
	struct Window *win;
	
	if (data->attached)
	{
		if (!data->attaching && (win = (struct Window *)getv(obj, MUIA_Window_Window)))
		{
			ULONG dir;
			ULONG dist, distcomp;
			ULONG tx = tx; /* shut up, gcc */
			ULONG ty = ty; /* really */
			struct Screen *scr = win->WScreen;
			ULONG was_offscreen = FALSE;

			/*
			 * If we are in offscreen mode, we need to
			 * put back the panels inside and ignore the rest.
			 */
			ty = win->TopEdge;
			tx = win->LeftEdge;

			if (win->TopEdge < 0)
			{
				ty = 0;
				was_offscreen = TRUE;
			}

			if (win->LeftEdge < 0)
			{
				tx = 0;
				was_offscreen = TRUE;
			}

			if (win->TopEdge + win->Height > scr->Height)
			{
				ty = scr->Height - win->Height;
				was_offscreen = TRUE;
			}

			if (win->LeftEdge + win->Width > scr->Width)
			{
				tx = scr->Width - win->Width;
				was_offscreen = TRUE;
			}
			
			/* XXX: we should check if a border is already set in the prefs! for example screenmode change.. the method might need a new parameter though */

			/*
			 * Find out to which border we
			 * have to attach.
			 */
			if (!was_offscreen)
			{
	            dir = BA_TOP;

				dist = win->TopEdge;
				distcomp = scr->Width - (win->LeftEdge + win->Width);
				
				if (distcomp < dist)
				{
					dir = BA_RIGHT;
					dist = distcomp;
				}
				distcomp = scr->Height - (win->TopEdge + win->Height);

				if (distcomp < dist)
				{
					dir = BA_BOTTOM;
					dist = distcomp;
				}
				distcomp = win->LeftEdge;

				if (distcomp < dist)
				{
					dir = BA_LEFT;
				}

				switch (dir)
				{
					case BA_LEFT:
						tx = 0;
						ty = win->TopEdge;
						break;

					case BA_RIGHT:
						tx = scr->Width - win->Width;
						ty = win->TopEdge;
						break;

					case BA_TOP:
						tx = win->LeftEdge;
						ty = 0;
						break;

					case BA_BOTTOM:
						tx = win->LeftEdge;
						ty = scr->Height - win->Height;
						break;
				}

				/*
				 * Find if we are near a corner, if so,
				 * attach to it.
				 */
				if (win->LeftEdge < scr->Width * CORNER_ATTRACTION / 100)
				{
					tx = 0;
				}
				if (win->TopEdge < scr->Height * CORNER_ATTRACTION / 100)
				{
					ty = 0;
				}
				if (scr->Width - (win->LeftEdge + win->Width) < scr->Width * CORNER_ATTRACTION / 100)
				{
					tx = scr->Width - win->Width;
				}
				if (scr->Height - (win->TopEdge + win->Height) < scr->Height * CORNER_ATTRACTION / 100)
				{
					ty = scr->Height - win->Height;
				}

				setprefslong_ctx(data->pctx, DSI_PANELGROUP_BORDERATTACH, dir);
			}

			do_action(obj, TA_Panels_Move,
				TT_Panels_Move_Window, win,
				TT_Panels_Move_X, tx,
				TT_Panels_Move_Y, ty,
			TAG_DONE);


			data->attaching = TRUE;
		}
	}

	if (!data->attached || msg->immediate_save)
	{
		/* save the new positions immediately */
		DoMethod(obj, MM_Panelwin_SaveConfig);
	}
	return (0);
}

DEFSMETHOD(Panelwin_FindObjectByType)
{
	GETDATA;

	FORCHILD(data->grp, MUIA_Group_ChildList)
	{
		if (getv(child, MA_Panel_Type) == msg->type)
			return (ULONG)child;
	}
	NEXTCHILD

	return (0);
}

#if USE_TRAY_LIB

DEFSMETHOD(Panelwin_DetachTrayObject)
{
	GETDATA;

	FORCHILD(data->grp, MUIA_Group_ChildList)
	{
		if (getv(child, MA_Panel_Type) == MV_Panel_Type_TrayPanel)
		{
			return DoMethod(child, MM_TrayPanel_DetachObject, msg->object);
		}
	}
	NEXTCHILD

	return (FALSE);
}

#endif

DEFSMETHOD(Thread_Finished)
{
	GETDATA;

	if (msg->action == TA_Panels_Move)
	{
		data->attaching = FALSE;

		if (msg->status)
		{
			/* save the new positions now */
			DoMethod(obj, MM_Panelwin_SaveConfig);
		}
	}
	else if (!msg->action || msg->action == TA_Panels_Delete) /* XXX: hm.. a bit messy.. use a switch() case */
	{
		DoMethod(app, OM_REMMEMBER, obj);
		MUI_DisposeObject(obj);
	}
	return (0);
}

DEFMMETHOD(Window_Setup)
{
	GETDATA;
	/*
	 * Set proper window mode here.
	 */
	ASSERT(data->grp);

	switch (getv(data->grp, MA_Panelgroup_Depth))
	{
		case MV_Panelgroup_Depth_Normal:
			SetAttrs(obj, MUIA_Window_Backdrop, FALSE, MUIA_Window_Frontdrop, FALSE, TAG_DONE);
			break;

		case MV_Panelgroup_Depth_Back:
			SetAttrs(obj, MUIA_Window_Backdrop, TRUE, MUIA_Window_Frontdrop, FALSE, TAG_DONE);
			break;

		case MV_Panelgroup_Depth_Front:
			SetAttrs(obj, MUIA_Window_Backdrop, FALSE, MUIA_Window_Frontdrop, EnableExtensions ? TRUE : FALSE, TAG_DONE);
			break;

		#ifdef DEBUG
		default:
			PDB(("wrong val\n"));
			break;
		#endif
	}

	/*
	 * If the user changed the screenmode this lets
	 * attached panels join their borders.
	 */
	if (data->attached)
	{
		DoMethod(app, MUIM_Application_PushMethod, obj, 1, MM_Panelwin_AttachToBorder, FALSE);
	}

	return (DOSUPER);
}


DEFMMETHOD(Window_Cleanup)
{
	GETDATA;

    return (DOSUPER);
}


BEGINMTABLE
DECNEW
DECGET
DECSET
DECTMETHOD(Panelwin_CloseConfigwin)
DECTMETHOD(Panelwin_Close)
DECSMETHOD(Panelwin_Delete_Ok)
DECTMETHOD(Panelwin_SaveConfig)
DECSMETHOD(Panelwin_AttachToBorder)
DECSMETHOD(Panelwin_FindObjectByType)
#if USE_TRAY_LIB
DECSMETHOD(Panelwin_DetachTrayObject)
#endif
DECSMETHOD(Thread_Finished)
DECMMETHOD(Window_Setup)
DECMMETHOD(Window_Cleanup)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Window, panelwinclass)

